@@ -33,7 +33,6 @@ from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import units
-import pyudev
 import requests
 import tenacity
 
@@ -59,22 +58,8 @@ CONF = cfg.CONF
 AGENT_PARAMS_CACHED = dict()
 
 
-LSBLK_COLUMNS = ['KNAME', 'MODEL', 'SIZE', 'ROTA', 'TYPE', 'UUID', 'PARTUUID']
-
-
-COLLECT_LOGS_COMMANDS = {
-    'ps': ['ps', 'au'],
-    'df': ['df', '-a'],
-    'iptables': ['iptables', '-L'],
-    'ip_addr': ['ip', 'addr'],
-    'lshw': ['lshw', '-quiet', '-json'],
-    'lsblk': ['lsblk', '--all', '-o%s' % ','.join(LSBLK_COLUMNS)],
-    'lsblk-full': ['lsblk', '--all', '--bytes', '--output-all', '--pairs'],
-    'mdstat': ['cat', '/proc/mdstat'],
-    'mount': ['mount'],
-    'parted': ['parted', '-l'],
-    'multipath': ['multipath', '-ll'],
-}
+LSBLK_COLUMNS = ['KNAME', 'MODEL', 'SIZE', 'ROTA',
+                 'TYPE', 'UUID', 'PARTUUID', 'SERIAL']
 
 
 DEVICE_EXTRACTOR = re.compile(r'^(?:(.*\d)p|(.*\D))(?:\d+)$')
@@ -239,7 +224,7 @@ def _find_mount_point(device):
 def _check_vmedia_device(vmedia_device_file):
     """Check if a virtual media device appears valid.
 
-    Explicitly ignores partitions, actual disks, and other itmes that
+    Explicitly ignores partitions, actual disks, and other items that
     seem unlikely to be virtual media based items being provided
     into the running operating system via a BMC.
 
@@ -539,40 +524,12 @@ def gzip_and_b64encode(io_dict=None, file_list=None):
         return _encode_as_text(fp.getvalue())
 
 
-def _collect_udev(io_dict):
-    """Collect device properties from udev."""
+def try_collect_command_output(io_dict, file_name, command):
+    LOG.debug('Collecting command output: %s', command)
     try:
-        out, _e = ironic_utils.execute('lsblk', '-no', 'KNAME')
-    except processutils.ProcessExecutionError as exc:
-        LOG.warning('Could not list block devices: %s', exc)
-        return
-
-    context = pyudev.Context()
-
-    for kname in out.splitlines():
-        kname = kname.strip()
-        if not kname:
-            continue
-
-        name = os.path.join('/dev', kname)
-
-        try:
-            udev = pyudev.Devices.from_device_file(context, name)
-        except Exception as e:
-            LOG.warning("Device %(dev)s is inaccessible, skipping... "
-                        "Error: %(error)s", {'dev': name, 'error': e})
-            continue
-
-        try:
-            props = dict(udev.properties)
-        except AttributeError:  # pyudev < 0.20
-            props = dict(udev)
-
-        fp = io.TextIOWrapper(io.BytesIO(), encoding='utf-8')
-        json.dump(props, fp)
-        buf = fp.detach()
-        buf.seek(0)
-        io_dict[f'udev/{kname}'] = buf
+        io_dict[file_name] = get_command_output(command)
+    except errors.CommandExecutionError:
+        LOG.debug('Collecting logs from command %s has failed', command)
 
 
 def collect_system_logs(journald_max_lines=None):
@@ -588,12 +545,6 @@ def collect_system_logs(journald_max_lines=None):
     """
     LOG.info('Collecting system logs and debugging information')
 
-    def try_get_command_output(io_dict, file_name, command):
-        try:
-            io_dict[file_name] = get_command_output(command)
-        except errors.CommandExecutionError:
-            LOG.debug('Collecting logs from command %s has failed', command)
-
     io_dict = {}
     file_list = []
     log_locations = [CONF.log_file, CONF.log_dir]
@@ -603,20 +554,19 @@ def collect_system_logs(journald_max_lines=None):
             if log_loc and os.path.exists(log_loc):
                 file_list.append(log_loc)
     else:
-        try_get_command_output(io_dict, 'dmesg', ['dmesg'])
         file_list.append('/var/log')
         for log_loc in log_locations:
             if (log_loc and os.path.exists(log_loc)
                     and not log_loc.startswith('/var/log')):
                 file_list.append(log_loc)
 
-    for name, cmd in COLLECT_LOGS_COMMANDS.items():
-        try_get_command_output(io_dict, name, cmd)
-
+    # Avoid circular imports
+    from ironic_python_agent import hardware
     try:
-        _collect_udev(io_dict)
-    except Exception:
-        LOG.exception('Unexpected error when collecting udev properties')
+        hardware.dispatch_to_all_managers('collect_system_logs',
+                                          io_dict, file_list)
+    except errors.HardwareManagerMethodNotFound:
+        LOG.warning('All hardware managers failed to collect logs')
 
     return gzip_and_b64encode(io_dict=io_dict, file_list=file_list)
 
@@ -651,12 +601,28 @@ def extract_device(part):
     return (m.group(1) or m.group(2))
 
 
+def split_device_and_partition_number(part):
+    """Extract the partition number from a partition name or path.
+
+    :param part: the partition
+    :return: device and partition number if success, None otherwise
+    """
+
+    device = extract_device(part)
+    if not device:
+        return None
+    partition_number = part.replace(device, '')
+    if 'nvme' in device and partition_number[0] == 'p':
+        partition_number = partition_number[1:]
+    return (device, partition_number)
+
+
 # See ironic.drivers.utils.get_node_capability
 def _parse_capabilities_str(cap_str):
     """Extract capabilities from string.
 
     :param cap_str: string meant to meet key1:value1,key2:value2 format
-    :return: a dictionnary
+    :return: a dictionary
     """
     LOG.debug("Parsing capability string %s", cap_str)
     capabilities = {}
@@ -723,7 +689,7 @@ def get_node_boot_mode(node):
     'instance_info/capabilities' of node. Otherwise it directly look for boot
     mode hints into
 
-    :param node: dictionnary.
+    :param node: dictionary.
     :returns: 'bios' or 'uefi'
     """
     instance_info = node.get('instance_info', {})
@@ -731,7 +697,7 @@ def get_node_boot_mode(node):
     node_caps = parse_capabilities(node.get('properties', {}))
 
     if _is_secure_boot(instance_info_caps, node_caps):
-        LOG.debug('Deploy boot mode is implicitely uefi for because secure '
+        LOG.debug('Deploy boot mode is implicitly uefi because secure '
                   'boot is activated.')
         return 'uefi'
 
@@ -804,7 +770,7 @@ def determine_time_method():
 
     :returns: "ntpdate" if ntpdate has been found, "chrony" if chrony
               was located, and None if neither are located. If both tools
-              are present, "chrony" will supercede "ntpdate".
+              are present, "chrony" will supersede "ntpdate".
     """
     try:
         execute('chronyd', '-h')
@@ -951,3 +917,30 @@ def rescan_device(device):
     except processutils.ProcessExecutionError as e:
         LOG.warning('Something went wrong when waiting for udev '
                     'to settle. Error: %s', e)
+
+
+def _lshw_matches(item, by_id, fields):
+    lshw_id = item.get('id', '')
+    if isinstance(by_id, re.Pattern):
+        if by_id.match(lshw_id) is None:
+            return False
+    elif by_id is not None and by_id != lshw_id:
+        return False
+
+    for key, value in fields.items():
+        if item.get(key) != value:
+            return False
+
+    return True
+
+
+def find_in_lshw(lshw, by_id=None, by_class=None, recursive=False, **fields):
+    """Yield all suitable records from lshw."""
+    # Cannot really pass class=... in Python
+    if by_class is not None:
+        fields['class'] = by_class
+    for child in lshw.get('children', ()):
+        if _lshw_matches(child, by_id, fields):
+            yield child
+        if recursive:
+            yield from find_in_lshw(child, by_id, recursive=True, **fields)
